// Bloodymad 07/2003

#include <afx.h>
#include "types.h"

#pragma pack( push, 1 )
struct VZLocalFileHeader {
	enum { SIGNATURE=0x04034b50 };
    
	uint32 signature;
	uint16 minVersion;
	uint16 flags;
	uint16 compressionMethod;
	uint32 lastFileMod; // time|date -> FileTimeToDosDateTime
	uint32 crc32;
	uint32 compressedSize;
	uint32 uncompressedSize;
	uint16 filenameLength;
	uint16 extrafieldLength;
	// filename
};

struct VZCentralDirectory {
	enum { SIGNATURE=0x02014b50 };
    
	uint32 signature;
	uint16 version;
	uint16 minVersion;
	uint16 flags;
	uint16 compressionMethod;
	uint32 lastFileMod;
	uint32 crc32;
	uint32 compressedSize;
	uint32 uncompressedSize;
	uint16 filenameLength;
	uint16 extrafieldLength;
	uint16 commentLength;
	uint16 disknumber;
	uint16 internalFileAttributes;
	uint32 externalFileAttributes; // 0 = ReadOnly. 1 = Hidden. 2 = System.
	uint32 localHeaderOffset;
	// filename
};

struct VZCentralDirectoryEnd {
	enum { SIGNATURE=0x06054b50 };

	uint32 signature;
	uint16 diskNo;
	uint16 diskCD;
	uint16 diskEntries;
	uint16 globalEntries;
	uint32 cdLength;
	uint32 cdOffset;
	uint16 commentLength;
};
#pragma pack( pop )

class CVZSection {
protected:
	uint32 sectionoffset;
	bool sectionactive;
public:
	CVZSection();
	bool OffsetInSection(uint32 offset) const;
	uint32 Begin() const {return sectionoffset;};
	uint32 End() const {return sectionoffset+Size();};
	void Seek(uint32 lOff);
// virtual
public:
	virtual ~CVZSection();
	virtual void EnterSection();
	virtual void LeaveSection();
// virtual abstract
protected:
	virtual uint32 MaxRead() const = 0;
	virtual void LocalSeek(uint32 lOff) = 0;
public:
	virtual uint32 Read(void* lpBuf, uint32 nCount) = 0;
	virtual uint32 Size() const = 0;
};

class CFileInfo {
public:
	VZLocalFileHeader	ziplocalheader;
	VZCentralDirectory	zipcentraldir;
	char*	filename;
	CFileInfo();
	void FillLocalFileHeader();
};

class CVirtualZipFile : public CFile {
protected:
	CList<CVZSection*,CVZSection*>	*sections;
	uint32		size;
	uint32		position;
	POSITION	actsectionpos;
	CVZSection	*actsection;
	uint32		alignBlockSize;
	CString		*basedir;
	UINT		openFlags;

	void SwitchToSection(POSITION newsection);
public:
	CVirtualZipFile();
	CVirtualZipFile(LPCTSTR lpszDirName, UINT nOpenFlags, uint32 align = 1);
	~CVirtualZipFile();
	ULONGLONG GetPosition() const;
	CString GetFileName() const;

	BOOL Open(LPCTSTR lpszFileName, UINT nOpenFlags, CFileException* pError = NULL);

	CFile* Duplicate() const;

	ULONGLONG Seek(LONGLONG lOff, UINT nFrom);
	void SetLength(ULONGLONG dwNewLen);
	ULONGLONG GetLength() const;

	UINT Read(void* lpBuf, UINT nCount);
	void Write(const void* lpBuf, UINT nCount);

	void LockRange(ULONGLONG dwPos, ULONGLONG dwCount);
	void UnlockRange(ULONGLONG dwPos, ULONGLONG dwCount);

	void Abort();
	void Flush();
	void Close();
};